#!/bin/bash

# Copyright (c) 2019 Oracle and/or its affiliates. All rights reserved.

TUNED_LOG=/var/log/tuned/tuned.log
MY_TUNED_DIR=/var/lib/tuned/oci-rps-xps
SAVED_RX0_RPS_CPUS=$MY_TUNED_DIR/saved.rx-0.rps_cpus
SAVED_RX1_RPS_CPUS=$MY_TUNED_DIR/saved.rx-1.rps_cpus
SAVED_TX0_XPS_CPUS=$MY_TUNED_DIR/saved.tx-0.xps_cpus
SAVED_TX1_XPS_CPUS=$MY_TUNED_DIR/saved.tx-1.xps_cpus

if [[ -z "$MIN_NUM_CPUS" ]]; then
	MIN_NUM_CPUS=4
fi
if [[ -z "$MIN_NUM_VOLS" ]]; then
	MIN_NUM_VOLS=2
fi

# check if running on a virtual machine
vir_what=$(virt-what 2>/dev/null)

# the number of CPUs
num_cpus=$(lscpu --extended --online 2>/dev/null | grep yes | wc -l)

# the number of iSCSI-attached block volumes except for the boot volume
num_vols=$(iscsiadm -m session 2>/dev/null | grep -v boot | wc -l)

# the number of NICs, RX or TX queues
num_nics=$(ip addr | awk '/state UP/ {print $2}'| wc -l)
net_inf1=$(ip addr | awk '/state UP/ {print $2}'| sed 's/.$//'| head -1)
net_queues=/sys/class/net/$net_inf1/queues
num_rxqs=$(ls -ld /sys/class/net/$net_inf1/queues/rx-* | wc -l)
num_txqs=$(ls -ld /sys/class/net/$net_inf1/queues/tx-* | wc -l)

. /usr/lib/tuned/functions

log_info() {
	echo "`date +'%F %T,%3N'` INFO     tuned profile 'oci-rps-xps': $1" >> $TUNED_LOG
}

start() {
	if [[ ! -e $SAVED_RX0_RPS_CPUS ]]; then
		# first time being called
		if [[ ! -d $MY_TUNED_DIR ]]; then
			mkdir -p $MY_TUNED_DIR
		fi
		# save previous settings of RPS/XPS
		if [[ $num_rxqs -eq 1 ]] && [[ $num_txqs -eq 1 ]]; then
			cat $net_queues/rx-0/rps_cpus > $SAVED_RX0_RPS_CPUS
		elif [[ $num_rxqs -eq 2 ]] && [[ $num_txqs -eq 2 ]]; then
			cat $net_queues/rx-0/rps_cpus > $SAVED_RX0_RPS_CPUS
			cat $net_queues/rx-1/rps_cpus > $SAVED_RX1_RPS_CPUS
			cat $net_queues/tx-0/xps_cpus > $SAVED_TX0_XPS_CPUS
			cat $net_queues/tx-1/xps_cpus > $SAVED_TX1_XPS_CPUS
		fi
	fi

	if [[ $vir_what != 'kvm' ]]; then
		log_info "not on KVM, so RPS/XPS settings are not changed"
		return 0
	fi

	if [[ $num_nics -ne 1 ]]; then
		log_info "on KVM, $num_nics NICs, so RPS/XPS settings are not changed"
		return 0
	fi

	if [[ $num_cpus -ge $MIN_NUM_CPUS ]] && [[ $num_vols -ge $MIN_NUM_VOLS ]]; then
		if [[ $num_rxqs -eq 1 ]] && [[ $num_txqs -eq 1 ]]; then
			# enable RPS when having one RX queue and one TX queue
			echo 0 > $net_queues/rx-0/rps_cpus
			old_val=$(cat $net_queues/rx-0/rps_cpus)
			new_val=$(echo $old_val | sed 's/0/f/g')
			echo $new_val > $net_queues/rx-0/rps_cpus
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, RPS is enabled"
		elif [[ $num_rxqs -eq 2 ]] && [[ $num_txqs -eq 2 ]]; then
			# enable RPS and XPS when having two RX queues and two TX queues
			echo 0 > $net_queues/rx-0/rps_cpus
			old_val=$(cat $net_queues/rx-0/rps_cpus)
			new_val1=$(echo $old_val | sed 's/0/1/g')
			new_val2=$(echo $old_val | sed 's/0/2/g')
			new_val4=$(echo $old_val | sed 's/0/4/g')
			new_val8=$(echo $old_val | sed 's/0/8/g')
			echo $new_val1 > $net_queues/rx-0/rps_cpus
			echo $new_val2 > $net_queues/rx-1/rps_cpus
			echo $new_val4 > $net_queues/tx-0/xps_cpus
			echo $new_val8 > $net_queues/tx-1/xps_cpus
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, both RPS and XPS are enabled"
		else
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, RPS/XPS settings are not changed"
		fi
	else
		if [[ $num_rxqs -eq 1 ]] && [[ $num_txqs -eq 1 ]]; then
			# disable RPS and XPS when having one RX queue and one TX queue
			echo 0 > $net_queues/rx-0/rps_cpus
			echo 0 > $net_queues/tx-0/xps_cpus
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, both RPS and XPS are disabled"
		elif [[ $num_rxqs -eq 2 ]] && [[ $num_txqs -eq 2 ]]; then
			# disable RPS and XPS when having two RX queues and two TX queues
			echo 0 > $net_queues/rx-0/rps_cpus
			echo 0 > $net_queues/rx-1/rps_cpus
			echo 0 > $net_queues/tx-0/xps_cpus
			echo 0 > $net_queues/tx-1/xps_cpus
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, both RPS and XPS are disabled"
		else
			log_info "on KVM, $num_cpus CPUs, $num_vols volumes, RPS/XPS settings are not changed"
		fi
	fi

	return 0
}

stop() {
	# restore previous settings of RPS/XPS
	if [[ -e $SAVED_RX0_RPS_CPUS ]]; then
		cat $SAVED_RX0_RPS_CPUS > $net_queues/rx-0/rps_cpus
		rm -f $SAVED_RX0_RPS_CPUS
	fi
	if [[ -e $SAVED_RX1_RPS_CPUS ]]; then
		cat $SAVED_RX1_RPS_CPUS > $net_queues/rx-1/rps_cpus
		rm -f $SAVED_RX1_RPS_CPUS
	fi
	if [[ -e $SAVED_TX0_XPS_CPUS ]]; then
		cat $SAVED_TX0_XPS_CPUS > $net_queues/tx-0/xps_cpus
		rm -f $SAVED_TX0_XPS_CPUS
	fi
	if [[ -e $SAVED_TX1_XPS_CPUS ]]; then
		cat $SAVED_TX1_XPS_CPUS > $net_queues/tx-1/xps_cpus
		rm -f $SAVED_TX1_XPS_CPUS
	fi

	if [[ -d "$MY_TUNED_DIR" ]]; then
		rm -r $MY_TUNED_DIR
	fi

	return 0
}

process $@
